
# Copyright (C) Integrate.ai, Inc. All rights reserved.

import subprocess
import sys
from integrate_ai.utils.rest_client import RestClient
import typer
import os
import rich
from integrate_ai.utils.typer_utils import (
    TogglePromptOption,
)

app = typer.Typer(no_args_is_help=True)
# Default white list for AWS addr to query EC2 metadata
no_proxy_addr = "169.254.169.254,169.254.170.2,/var/run/docker.sock"


@app.command()
def install(
    token: str = TogglePromptOption(
        ...,
        help="Your generated IAI token.",
        prompt="Please provide your IAI token",
        envvar="IAI_TOKEN",
    ),
    taskrunner_name: str = TogglePromptOption(
        ...,
        help="The taskrunner name to register.",
        prompt="Please provide the taskrunner name",
        envvar="IAI_TASKRUNNER_NAME",
    ),
):
    """
    This command register this node and save the regiter information to ecsanywhere_output.txt
    """
    if os.geteuid() != 0:
        rich.print("This command must be run as root.")
        sys.exit(1)
    try:
        # clean up rm /tmp/ecs-anywhere-install.sh if there is any
        if os.path.exists("/tmp/ecs-anywhere-install.sh"):
            subprocess.run("rm /tmp/ecs-anywhere-install.sh", shell=True, check=True, text=True, capture_output=True)

        # register taskrunner id
        client = RestClient(token=token)
        response = client.register_on_prem_taskrunner(taskrunner_name)

        # install ecs anywhere
        rich.print("Installing ECS Agent..")
        install_ecs_agent(response)
        rich.print("---ECS Agent Installed---")
        taskrunners = client.get_taskrunner_info(taskrunner_name=taskrunner_name)
        prov_info = taskrunners[0].get("provisioning_info", {})
        runner_config = taskrunners[0].get("runner_config", {})
        runtime_info = prov_info.get("runtime_info")
        if runtime_info:
            proxy_addr = runtime_info.get("iai_forward_proxy_http_url")
            # configure proxy
            if proxy_addr and runner_config.get("use_global_proxy"):
                rich.print(f"Configuring ECS proxy: {proxy_addr}")
                configure_proxy(proxy_addr=proxy_addr)
        rich.print("OnPrem node installed successfully!")
    except Exception as e:
        rich.print(
            f"On-Prem register command failed with error: {e}, Logs can be found in ecsanywhere_output.txt and proxy_config.txt"
        )
        raise e


def install_ecs_agent(response):
    # install ecs anywhere
    cmd = 'sudo curl --proto "https" -o "/tmp/ecs-anywhere-install.sh" "https://amazon-ecs-agent.s3.amazonaws.com/ecs-anywhere-install-latest.sh"'
    cmd += " && sudo bash /tmp/ecs-anywhere-install.sh"
    cmd += f' --region "{response["region"]}"'
    cmd += f' --cluster "{response["cluster_name"]}"'
    cmd += f' --activation-id "{response["activation_id"]}"'
    cmd += f' --activation-code "{response["activation_code"]}"'
    cmd += "> ecsanywhere_output.txt"
    try:
        rich.print("Registering...")
        subprocess.run(cmd, shell=True, check=True, timeout=1200)
        rich.print("Agent registered successfully.")
        rich.print("Output is saved in ecsanywhere_output.txt. The file contains instance id, please do not delete.")
    except subprocess.CalledProcessError as e:
        message = f"ECS Installation command failed with error: {e.stderr}, Logs can be found in ecsanywhere_output.txt"
        rich.print(message)
        raise Exception(message) from e


# Add proxy addresses to the ecs config so traffic can go through the proxy by default
# See : https://docs.aws.amazon.com/AmazonECS/latest/developerguide/http_proxy_config.html
def configure_proxy(proxy_addr: str):
    rich.print("Configuring Proxy config in the host")
    # proxy ssm agent
    ssm_config_cmd = "sudo mkdir -p /etc/systemd/system/amazon-ssm-agent.service.d"
    ssm_config_cmd += "&& { cat > /etc/systemd/system/amazon-ssm-agent.service.d/http-proxy.conf <<-EOF \n"
    ssm_config_cmd += "[Service] \n"
    ssm_config_cmd += f"Environment=HTTPS_PROXY={proxy_addr} \n"
    ssm_config_cmd += f"Environment=NO_PROXY={no_proxy_addr} \n"
    ssm_config_cmd += "EOF\n"
    ssm_config_cmd += "}"

    # proxy docker service
    docker_svc_cmd = "sudo mkdir -p /etc/systemd/system/docker.service.d"
    docker_svc_cmd += "&& { cat > /etc/systemd/system/docker.service.d/http-proxy.conf <<-EOF \n"
    docker_svc_cmd += "[Service] \n"
    docker_svc_cmd += f"Environment=HTTPS_PROXY={proxy_addr} \n"
    docker_svc_cmd += f"Environment=NO_PROXY={no_proxy_addr} \n"
    docker_svc_cmd += "EOF\n"
    docker_svc_cmd += "}"

    # proxy ecs-init
    ecs_init_config_cmd = "sudo mkdir -p /etc/systemd/system/ecs.service.d"
    ecs_init_config_cmd += "&& { cat > /etc/systemd/system/ecs.service.d/http-proxy.conf <<-EOF \n"
    ecs_init_config_cmd += "[Service] \n"
    ecs_init_config_cmd += f"Environment=HTTPS_PROXY={proxy_addr} \n"
    ecs_init_config_cmd += f"Environment=NO_PROXY={no_proxy_addr} \n"
    ecs_init_config_cmd += "EOF\n"
    ecs_init_config_cmd += "}"

    # proxy ecs
    ecs_config_cmd = "echo patching ecs config"
    ecs_config_cmd += "&& { cat >> /etc/ecs/ecs.config <<-EOF \n"
    ecs_config_cmd += "[Service] \n"
    ecs_config_cmd += f"HTTPS_PROXY={proxy_addr} \n"
    ecs_config_cmd += f"NO_PROXY={no_proxy_addr} \n"
    ecs_config_cmd += "EOF\n"
    ecs_config_cmd += "}"

    # restart all affected services
    restart_cmd = (
        "sudo systemctl daemon-reload && sudo systemctl restart docker.service && sudo systemctl restart "
        "amazon-ssm-agent && sudo systemctl restart ecs"
    )

    # completion_command
    completion_command = "echo proxy config successfully updated"

    command_to_execute = f"{ssm_config_cmd} && {docker_svc_cmd} && {ecs_init_config_cmd} && {ecs_config_cmd} && {restart_cmd} && {completion_command}"
    try:
        # rich.print(f"Configuring Proxy command : {command_to_execute}")
        subprocess.run(command_to_execute, shell=True, check=True, timeout=1200)
        rich.print("Proxy configured successfully")
    except subprocess.CalledProcessError as e:
        message = f"Proxy configuration failed with error: {e.stderr}"
        rich.print(message)
        raise Exception(message) from e


@app.command()
def uninstall(
    token: str = TogglePromptOption(
        ...,
        help="Your generated IAI token.",
        prompt="Please provide your IAI token",
        envvar="IAI_TOKEN",
    ),
    taskrunner_name: str = TogglePromptOption(
        ...,
        help="The taskrunner id to register.",
        prompt="Please provide the taskrunner id",
        envvar="IAI_TASKRUNNER_NAME",
    ),
):
    """
    This command deregister this node and clean up the directory.
    """
    if os.geteuid() != 0:
        rich.print("This command must be run as root.")
        sys.exit(1)
    # register taskrunner id
    client = RestClient(token=token)
    response = client.deregister_on_prem_taskrunner(taskrunner_name=taskrunner_name, instance_id=get_instance_id())
    rich.print("Deregister instance ", response["containerInstance"]["ec2InstanceId"])

    # uninstall ecs anywhere
    stop_ecs = "sudo systemctl stop ecs amazon-ssm-agent"

    try:
        stop_ecs_out = subprocess.run(stop_ecs, shell=True, check=True, text=True, capture_output=True)
        rich.print(stop_ecs_out.stdout)

        # Check OS and remove packages
        os_type = None
        try:
            with open("/etc/os-release", "r") as f:
                for line in f:
                    if line.startswith("ID="):
                        os_type = line.strip().split("=")[1].replace('"', "").lower()
                        break
        except FileNotFoundError:
            raise Exception("Unable to detect OS. /etc/os-release not found.")

        if "centos" in os_type or "rhel" in os_type:
            uninstall_packages_cmd = "sudo yum remove -y amazon-ecs-init amazon-ssm-agent"
        elif "debian" in os_type or "ubuntu" in os_type:
            uninstall_packages_cmd = "sudo apt remove -y amazon-ecs-init amazon-ssm-agent"
        else:
            raise Exception("Unsupported OS for package removal")

        uninstall_out = subprocess.run(uninstall_packages_cmd, shell=True, check=True, text=True, capture_output=True)
        rich.print(uninstall_out.stdout)

        # Remove leftover directories
        remove_directories_cmd = (
            "sudo rm -rf /var/lib/ecs /etc/ecs /var/lib/amazon/ssm /var/log/ecs /var/log/amazon/ssm"
        )
        remove_directories_cmd += " && sudo rm -rf /etc/systemd/system/amazon-ssm-agent.service.d/http-proxy.conf"
        remove_directories_cmd += " && sudo rm -rf /etc/systemd/system/docker.service.d/http-proxy.conf"
        remove_directories_cmd += " && sudo rm -rf /etc/systemd/system/ecs.service.d/http-proxy.conf"
        remove_directories_cmd += " && sudo rm -rf /etc/ecs/ecs.config"

        subprocess.run(remove_directories_cmd, shell=True, check=True, text=True)
        rich.print("Leftover directories removed")

        # Remove instance id file
        remove_output_cmd = "sudo rm ecsanywhere_output.txt"
        subprocess.run(remove_output_cmd, shell=True, check=True, text=True)
        rich.print("ecsanywhere_output.txt removed")
        rich.print("proxy entries removed")

    except subprocess.CalledProcessError as e:
        rich.print(f"Command failed with error: {e.stderr}")


def get_instance_id():
    result = subprocess.check_output(
        "grep 'Container instance arn:' ecsanywhere_output.txt | sed 's#.*/##'", shell=True
    )
    instance_id = result.decode("utf-8").strip().strip('"')
    if not instance_id:
        rich.print(
            "[red]Error: Could not parse instance ID from ecsanywhere_output.txt. Please verify the file's contents."
        )
        raise Exception("Instance ID not found.")
    rich.print("Deregister instance with instance_id ", instance_id)
    return instance_id


@app.callback()
def main():
    """
    Sub command for managing on-prem taskrunners related operations.
    """




# Copyright (C) Integrate.ai, Inc. All rights reserved.

"""Module containing Typer sdk submodule functionality."""

import typer
from rich import print as rprint
from integrate_ai.utils.rest_client import RestClient
import subprocess
from integrate_ai.utils.error_handling import IntegrateAIException

from integrate_ai.utils.typer_utils import TogglePromptOption, show_package_version

app = typer.Typer(no_args_is_help=True)
package = "integrate-ai-sdk"


@app.command()
def install(
    token: str = TogglePromptOption(
        ...,
        help="The IAI token.",
        prompt="Please provide your IAI token",
        envvar="IAI_TOKEN",
    ),
    version: str = typer.Option("", "--version", "-v", help="The version of the sdk to download."),
):
    """
    Install the integrate_ai sdk package. Defaults to pinned version
    Installing a new sdk will override the existing one.
    Will automatically read from the IAI_TOKEN environment variable if set.
    """
    rest_client = RestClient(token)
    pip_install_command = rest_client.get_pip_install_command(version=version, package=package)["pipInstallCommand"]
    if f"pip install {package}" not in pip_install_command:
        raise IntegrateAIException(f" Not installing the right package {pip_install_command}")

    # pip will output the version number and give an error if the version is not found
    if version == "":
        rprint(f"Trying to install the pinned version of {package}.")
    else:
        rprint(f"Trying to install {package}=={version}.")
    subprocess.run(pip_install_command, check=True, shell=True)


@app.command()
def version():
    """
    The currently installed version of the sdk.
    """
    show_package_version(package)


@app.command()
def uninstall():
    """
    Uninstall the sdk and associated files and artifacts.
    """

    rprint(f"Uninstalling {package}")
    pip_uninstall_command = f"pip uninstall {package}"
    subprocess.run(pip_uninstall_command, check=True, shell=True)


@app.command()
def list(
    token: str = TogglePromptOption(
        ..., help="The IAI token.", prompt="Please provide your IAI token", envvar="IAI_TOKEN"
    ),
):
    """
    List the versions of the sdk available to be installed.
    """
    rest_client = RestClient(token)
    versions = rest_client.get_package_versions(package=package)

    if versions["package_name"] != package:
        raise IntegrateAIException("Not getting the right package. ")

    if len(versions["package_versions"]) == 0:
        rprint("No available version.")
    else:
        for version in versions["package_versions"]:
            rprint(version)


@app.callback()
def main():
    """
    Sub command for managing sdk related operations.
    """
    pass  # pragma: no cover




# Copyright (C) Integrate.ai, Inc. All rights reserved.

"""Module containing exception-related functionality."""

from requests import Response  # type: ignore


class IntegrateAIException(Exception):
    """The general exception for request and response errors."""


class DockerException(Exception):
    """The general exception for docker-related errors."""


def check_for_IntegrateAIException(response: Response):
    """Raises an IntegrateAIException from a response.

    If response has an error status code, raises a custom IntegrateAIException instead
      of the original Exception.

    Args:
        response (Response): HTTP response to check for error status and raise custom
          message for.

    Raises:
        IntegrateAIException: Customized IntegrateAI exception for the HTTP Exception.
    """

    try:
        response.raise_for_status()
    except Exception as e:
        data = response.json()
        if isinstance(data, dict) and "message" in data.keys():
            response_message = data.get("message")
        else:
            response_message = response.text
        raise IntegrateAIException(response_message) from e




# Copyright (C) Integrate.ai, Inc. All rights reserved.

"""Module containing top-level Typer app functionality."""

import subprocess  # noqa: F401 - needed for unittest patch
import typer
import integrate_ai.sdk as sdk
import integrate_ai.client as client
import integrate_ai.server as server
import integrate_ai.onprem_node as onprem_node
from integrate_ai.utils.typer_utils import show_package_version


app = typer.Typer(no_args_is_help=True)

app.add_typer(sdk.app, name="sdk")
app.add_typer(client.app, name="client")
app.add_typer(server.app, name="server")
app.add_typer(onprem_node.app, name="onprem_node")

package = "integrate-ai"


@app.callback()
def docs(
    no_prompt: bool = typer.Option(False, "--no-prompt", help="Disable prompts. Confirmations will default to `Y`.")
):
    """
    The CLI interface to manage integrate.ai operations.

    Your IAI token will be required in many commmands. For convenience, you can set this in the `IAI_TOKEN` environment variable to avoid repeated prompts.
    """


@app.command()
def version():
    """
    The currently installed version of the cli.
    """
    show_package_version(package)


def main():

    app()


if __name__ == "__main__":
    main()  # pragma: no cover




# DO NOT EDIT THIS FILE -- AUTOGENERATED BY PANTS
# Target: integrate_ai:dist

from setuptools import setup

setup(**{
    'author': 'integrate.ai',
    'classifiers': [
        'Programming Language :: Python :: 3',
    ],
    'description': 'integrate.ai',
    'entry_points': {
        'console_scripts': [
            'iai = integrate_ai.cli:main',
        ],
    },
    'install_requires': (
        'docker==7.1.0',
        'pyjwt>=2.4.0',
        'requests<3.0.0,>=2.32.2',
        'rich<14.0.0,>=13.3.4',
        'typer<0.8.0,>=0.7.0',
    ),
    'license': 'Copyright (C) 2022 integrate.ai, Inc. All rights reserved.',
    'long_description': """# integrate.ai Command Line Interface

A CLI that enables users to interact with integrate.ai's software.

This tool allows users to manage client and sdk packages. Use the integate.ai sdk to seamlessly integrate federated learning and analytics workflows directly into your product.

## Installation

integrate-ai requires `Python >= 3.8`

```
pip install integrate-ai
```

## Quick Start

Create a new IAI token through the integrate.ai UI.

### Install integrate-ai-sdk

```
iai sdk install --token <IAI_TOKEN> 
```

### Pull integrate.ai docker client

```
iai client pull --token <IAI_TOKEN> 
```

### Run integrate.ai training session

```
iai client train --token <IAI_TOKEN> --session <SESSION_ID> --train_path <PATH_TO_TRAINING_DATA> --test_path <PATH_TO_TEST_DATA> --batch_size 5
```
""",
    'long_description_content_type': 'text/markdown',
    'name': 'integrate_ai',
    'namespace_packages': (
    ),
    'package_data': {
    },
    'packages': (
        'integrate_ai',
        'integrate_ai.utils',
    ),
    'python_requires': '>=3.7.5',
    'version': '9.22.5',
})




# Copyright (C) Integrate.ai, Inc. All rights reserved.

import os


def get_mounted_path(mount_path, path):
    if path.get("s3"):
        # No mount, pass the URL as is
        mounted_path = path["url"]
    else:
        mounted_path = path["full_path"].replace(path["parent_path"], mount_path + path["parent_dir"])
    return mounted_path


def get_volumes(mount_path, path):
    volumes = {}
    if not path.get("s3"):
        volumes[path["parent_path"]] = {
            "bind": mount_path + path["parent_dir"],
            "mode": "ro",
        }
    return volumes


def get_aws_env(path):
    if path.get("s3"):
        return {
            "AWS_ACCESS_KEY_ID": os.getenv("AWS_ACCESS_KEY_ID"),
            "AWS_SECRET_ACCESS_KEY": os.getenv("AWS_SECRET_ACCESS_KEY"),
            "AWS_SESSION_TOKEN": os.getenv("AWS_SESSION_TOKEN"),
            "AWS_REGION": os.getenv("AWS_REGION"),
        }
    return {}
